@@ -13,7 +13,7 @@
 #makes things work when run without install
 use lib qw( ../../perl-shared/blib/lib ../../perl-shared/blib/arch );
 # this is for after install
-use lib qw( /usr/local/rrdtool-1.0.48/lib/perl ../lib/perl );
+use lib qw( /scratch/oetiker/rrd/lib/perl ../lib/perl );
 
 use RRDs;
 
