@@ -556,6 +556,7 @@ tod(struct rrd_time_value *ptv)
 {
     int hour, minute = 0;
     int tlen;
+    
     /* save token status in  case we must abort */
     int scc_sv = scc; 
     char *sct_sv = sct; 
@@ -660,6 +661,7 @@ day(struct rrd_time_value *ptv)
 {
     long mday=0, wday, mon, year = ptv->tm.tm_year;
     int tlen;
+    time_t montime;
 
     switch (sc_tokid) {
     case YESTERDAY:
@@ -713,7 +715,8 @@ day(struct rrd_time_value *ptv)
 	    tlen = strlen(sc_token);
 	    mon = atol(sc_token);
             if (mon > 10*365*24*60*60) {
-		ptv->tm=*localtime(&mon);
+                montime = mon;
+                ptv->tm=*localtime(&montime);
 		token();
 		break;
 	    }
