@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.48  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.49  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
  * rrd_cgi.c  RRD Web Page Generator
  *****************************************************************************/
@@ -10,8 +10,8 @@
 
 
 #define MEMBLK 1024
-/*#define DEBUG_PARSER
-#define DEBUG_VARS*/
+/* #define DEBUG_PARSER */
+/* #define DEBUG_VARS */
 
 /* global variable for libcgi */
 s_cgi **cgiArg;
@@ -220,6 +220,9 @@ rrd_expand_vars(char* buffer)
 		parse(&buffer, i, "<RRD::CV::PATH", cgigetqp);
 		parse(&buffer, i, "<RRD::GETENV", rrdgetenv);	 
 		parse(&buffer, i, "<RRD::GETVAR", rrdgetvar);	 
+		parse(&buffer, i, "<RRD::TIME::LAST", printtimelast);
+		parse(&buffer, i, "<RRD::TIME::NOW", printtimenow);
+		parse(&buffer, i, "<RRD::TIME::STRFTIME", printstrftime);
 	}
 	return buffer;
 }
@@ -297,10 +300,6 @@ int main(int argc, char *argv[]) {
 	/* make sure we have one extra argument, 
 	   if there are others, we do not care Apache gives several */
 
-	/* if ( (optind != argc-2 
-	   && strstr( getenv("SERVER_SOFTWARE"),"Apache/2") != NULL) 
-	   && optind != argc-1) { */
-
 	if ( optind >= argc ) { 
 		fprintf(stderr, "ERROR: expected a filename\n");
 		exit(1);
@@ -316,6 +315,11 @@ int main(int argc, char *argv[]) {
 	/* initialize variable heap */
 	initvar();
 
+#ifdef DEBUG_PARSER
+	/* some fake header for testing */
+	printf ("Content-Type: text/html\nContent-Length: 10000000\n\n\n");
+#endif
+
 	/* expand rrd directives in buffer recursivly */
 	for (i=0; buffer[i]; i++) {
 		if (buffer[i] != '<')
@@ -611,57 +615,60 @@ char* cgigetq(long argc, char **args){
 
 /* remove occurrences of .. this is a general measure to make
    paths which came in via cgi do not go UP ... */
+char* cgigetqp(long argc, char **args)
+{
+	char* buf;
+    char* buf2;
+    char* p;
+	char* d;
 
-char* cgigetqp(long argc, char **args){
-  if (argc>= 1) {
-    char *buf = rrdstrip(cgiGetValue(cgiArg,args[0]));
-    char *buf2;
-    char *c,*d;
-    int  qc=0;
-
-    if (buf==NULL) 
-		return NULL;
-
-    for(c=buf;*c != '\0';c++) {
-    	if (*c == '"') {
-			qc++;
-		}
+	if (argc < 1)
+	{
+  		return stralloc("[ERROR: not enough arguments for RRD::CV::PATH]");
 	}
 
-    if ((buf2 = malloc((strlen(buf) + 4 * qc + 4))) == NULL) {
-		perror("Malloc Buffer");
+	buf = rrdstrip(cgiGetValue(cgiArg, args[0]));
+    if (!buf)
+	{
+		return NULL;
+	}
+
+	buf2 = malloc(strlen(buf)+1);
+    if (!buf2)
+	{
+		perror("cgigetqp(): Malloc Path Buffer");
 		exit(1);
     };
 
-    c=buf;
-    d=buf2;
+    p = buf;
+    d = buf2;
 
-    *(d++) = '"';
-    while (*c != '\0') {
-		if (*c == '"') {
-			*(d++) = '"';
-			*(d++) = '\'';
-			*(d++) = '"';
-			*(d++) = '\'';
+    while (*p)
+	{
+		/* prevent mallicious paths from entering the system */
+		if (p[0] == '.' && p[1] == '.')
+		{
+			p += 2;
+			*d++ = '_';
+			*d++ = '_';	
 		}
-		if(*c == '/') {
-			*(d++) = '_';
-			c++;
-		} else {
-			if (*c=='.' && *(c+1) == '.') {
-				c += 2;
-				*(d++) = '_'; *(d++) ='_';	
-			} else {
-				*(d++) = *(c++);
-			}
+		else
+		{
+			*d++ = *p++;
 		}
     }
-    *(d++) = '"';
-    *(d) = '\0';
+
+    *d = 0;
     free(buf);
+
+    /* Make sure the path is relative, e.g. does not start with '/' */
+    p = buf2;
+    while ('/' == *p)
+	{
+	    *p++ = '_';
+    }
+
     return buf2;
-  }
-  return stralloc("[ERROR: not enough arguments for RRD::CV::PATH]");
 }
 
 
@@ -825,7 +832,7 @@ scanargs(char *line, int *argument_count, char ***arguments)
 		{
 		case ' ': 
 			if (Quote || tagcount) {
-				/* copy quoted/tagged string */
+				/* copy quoted/tagged (=RRD expanded) strings */
 				*putP++ = c;
 			}
 			else if (in_arg)
@@ -851,7 +858,7 @@ scanargs(char *line, int *argument_count, char ***arguments)
 				}
 			} else {
 				if (!in_arg) {
-					/* reference argument string in argument array */
+					/* reference start of argument string in argument array */
 					argv[argc++] = putP;
 					in_arg=1;
 				}
@@ -860,22 +867,20 @@ scanargs(char *line, int *argument_count, char ***arguments)
 			break;
 
 		default:
-			if (!Quote) {
-				if (!in_arg) {
-					/* start new argument */
-					argv[argc++] = putP;
-					in_arg = 1;
+			if (!in_arg) {
+				/* start new argument */
+				argv[argc++] = putP;
+				in_arg = 1;
+			}
+			if (c == '>') {
+				if (tagcount) {
+					tagcount--;
 				}
-				if (c == '>') {
-					if (tagcount) {
-						tagcount--;
-					}
-				}
-				if (c == '<') {
-					tagcount++;
-					if (0 == strncmp(getP, "RRD::", strlen("RRD::"))) {
-						curarg_contains_rrd_directives = 1;
-					}
+			}
+			if (c == '<') {
+				tagcount++;
+				if (0 == strncmp(getP, "RRD::", strlen("RRD::"))) {
+					curarg_contains_rrd_directives = 1;
 				}
 			}
 			*putP++ = c;
@@ -940,7 +945,7 @@ parse(
 	char *(*func)(long argc, char **args) /* function to call for 'tag' */
 	)
 {
-	/* the name of the vairable ... */
+	/* the name of the variable ... */
 	char *val;
 	long valln;  
 	char **args;
