@@ -1,6 +1,6 @@
 
 /****************************************************************************
- * RRDtool 1.0.48  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.49  Copyright Tobias Oetiker, 1997 - 2000
  ****************************************************************************
  * rrd__graph.c  make creates ne rrds
  ****************************************************************************/
@@ -2966,7 +2966,7 @@ im.unit = 0; /* do not display unit if not set */
 
 	
 	opt = getopt_long(argc, argv, 
-			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:zgjFYAMXL:S:RB:O:U:",
+			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:zgjFYAMX:L:S:RB:O:U:",
 			  long_options, &option_index);
 
 	if (opt == EOF)
